#ifndef ATLAS_THREADS_H
   #define ATLAS_THREADS_H

#ifndef ATL_PTMAXMALLOC
   #ifndef ATL_PTMAXMALLOC_MB
      #define ATL_PTMAXMALLOC_MB 128
   #endif
   #define ATL_PTMAXMALLOC (((size_t)(ATL_PTMAXMALLOC_MB)<<20)>>ATL_NTHRPOW2)
#endif
#ifdef ATL_OS_WinNT
   #define ATL_WINTHREADS
#endif
#include "atlas_pthreads.h" /* gened file defs ATL_NTHREADS & ATL_NTHRPOW2 */
#ifdef ATL_WINTHREADS
   #include <windows.h>
   typedef struct
   {
      void *vp;      /* ptr to extra info */
      HANDLE thrH;   /* handle to thread */
      int rank;      /* my rank */
   } ATL_thread_t;
   #ifndef CREATE_SUSPENDED
      #define CREATE_SUSPENDED 0x00000004
   #endif
   #ifndef WAIT_FAILED
      #define WAIT_FAILED (~0)
   #endif
#else
   #include <pthread.h>
   typedef struct
   {
      pthread_t thrH;/* handle of thread */
      void *vp;      /* ptr to extra info */
      int rank;      /* my rank */
      #ifdef ATL_PAFF_SELF
         int paff_set;  /* have I set my process affinity yet? */
      #endif
   } ATL_thread_t;
#endif

typedef struct ATL_LaunchStruct ATL_LAUNCHSTRUCT_t;
struct ATL_LaunchStruct
{
   ATL_thread_t *rank2thr;              /* index by rank to get thread handle */
   char *opstruct;
   int (*OpStructIsInit)(void*);        /* Query to see if struct is valid */
   void (*CombineOpStructs)(void*, void*);  /* combine function */
   void (*DoWork)(ATL_LAUNCHSTRUCT_t*, void*);
   int opstructstride;                  /* size of operation-spec struct */
   volatile int *imdone;                /* nthr-len checkin array */
   void **acounts;                      /* var-len array of atomic counters */
   void *vp;                            /* misc. extra info ptr */
};


/* Sets up ATL_LAUNCHSTRUCT_t var and ATL_thread_t array & starts threads*/
void ATL_thread_launch(void *opstruct, int opstructstride, void *OpStructIsInit,
                       void *DoWork, void *CombineOpStructs);
/*  Starts a thread running, and sets its affinity to proc if possible */
int ATL_thread_start(ATL_thread_t *thr, int proc, void *(*rout)(void*), void*);
int ATL_thread_join(ATL_thread_t*); /* waits on completion of thread */
void ATL_thread_exit(void*);        /* exits currently executing thread */
void *ATL_log2tlaunch(void *vp);    /* min spanning tree launch */
void *ATL_lin0tlaunch(void *vp);    /* 0 linear launches all threads */
void *ATL_dyntlaunch(void *vp);     /* launch done as workpool */

void *ATL_SetAtomicCount(int cnt);  /* allocates acnt, sets acnt=cnt */
int   ATL_DecAtomicCount(void *vp); /* returns acnt-- (not --acnt!) */
int   ATL_GetAtomicCount(void *vp); /* returns acnt */
void ATL_FreeAtomicCount(void *vp); /* free acount resources */

#define MindxT(A_,i_) ((void*)( ((char*)(A_)) + ((size_t)i_) ))
#define ATL_tlaunch ATL_log2tlaunch   /* may want linear launch later */

#endif   /* end of #ifdef protecting include file from redundant inclusion */

